package com.jz.dmp.modules.service.impl;

import com.alibaba.fastjson.JSON;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jz.agent.service.DmpDsAgentService;
import com.jz.common.constant.JsonResult;
import com.jz.common.constant.ResultCode;
import com.jz.common.page.BasePageBean;
import com.jz.common.page.PageInfoResponse;
import com.jz.common.persistence.BaseService;
import com.jz.common.utils.AzkabanApiUtils2;
import com.jz.common.utils.FileUtils;
import com.jz.common.utils.JsonMapper;
import com.jz.common.utils.ZipUtils;
import com.jz.dmp.agent.DmpAgentResult;
import com.jz.dmp.modules.controller.DataIntegration.bean.*;
import com.jz.dmp.modules.controller.DataIntegration.bean.flow.FlowExecution;
import com.jz.dmp.modules.dao.*;
import com.jz.dmp.modules.model.*;
import com.jz.dmp.modules.service.OfflineSynchService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @Description:离线同步服务层
 * @ClassName: OfflineSynchServiceImpl
 * @Author Bellamy
 * @Date 2020/12/20
 */

@Service("offlineSynchService")
@Transactional
public class OfflineSynchServiceImpl implements OfflineSynchService {

    private static Logger logger = LoggerFactory.getLogger(OfflineSynchServiceImpl.class);

    @Value("${spring.public-key}")
    private String publicKey;

    @Autowired
    private OfflineSynchDao offlineSynchDao;

    @Autowired
    private DmpProjectDao dmpProjectDao;

    @Autowired
    private DmpDevelopTaskDao dmpDevelopTaskDao;

    @Autowired
    private DmpDsAgentService dmpDsAgentServiceImp;

    @Autowired
    private DmpTableDao dmpTableDao;

    @Autowired
    private DmpTableFieldSchemaDao dmpTableFieldSchemaDao;

    @Autowired
    private DmpTableColumnDao dmpTableColumnDao;

    @Autowired
    private DvRuleTDao dvRuleTDao;

    @Override
    public PageInfoResponse<TaskListPageDto> queryTaskListPage(TaskListPageReq taskListPageReq) throws Exception {
        PageInfoResponse<TaskListPageDto> pageInfoResponse = new PageInfoResponse<>();

        Map param = new HashMap();
        //param.put("userId", user.getUserId());
        //param.put("pageNum", pageBean.getPageNum());
        //param.put("pageSize", pageBean.getPageSize());
        PageHelper.startPage(taskListPageReq.getPageNum(), taskListPageReq.getPageSize());
        List<TaskListPageDto> list = offlineSynchDao.queryTaskListPage(taskListPageReq);
        PageInfo<TaskListPageDto> pageInfo = new PageInfo<>(list);

        pageInfoResponse.setCode(ResultCode.SUCCESS);
        pageInfoResponse.setMessage("查询成功");
        pageInfoResponse.setData(pageInfo);
        return pageInfoResponse;
    }

    /**
     * 获取源数据库名称——下拉框
     */
    @Override
    public JsonResult<List<SourceDbNameListDto>> querygSourceDbList(Integer projectId) throws Exception {
        Map map = new HashMap();
        map.put("projectId", projectId); //项目id
        map.put("isEnableSource", "1");
        List<SourceDbNameListDto> list = offlineSynchDao.querygSourceDbList(map);
        return new JsonResult(ResultCode.SUCCESS, list);
    }

    /**
     * 根据源数据库id,获取源数据表——下拉框
     */
    @Override
    public JsonResult querygSourceTableList(Long sourceDbId, String targetName) throws Exception {
        //通过源数据库id ，查询数据源配置
        DmpAgentDatasourceInfo dsInfo = offlineSynchDao.querySourceDbInfoBySourceId(sourceDbId);
        //解码源数据库密码
        if (StringUtils.isNotBlank(dsInfo.getPassword())) {
            dsInfo.setPassword(new BaseService().decode(dsInfo.getPassword(), publicKey));
        }
        if (StringUtils.isNotBlank(targetName)) {
            dsInfo.setTargetFileName(targetName);
        }
        //创建jdbc,获取源数据表
        DmpAgentResult rst = dmpDsAgentServiceImp.getTableNameList(dsInfo);
        if (!rst.getCode().val().equals("200")) {
            return new JsonResult(rst.getCode(), rst.getMessage());
        } else {
            //成功
            rst.setResult(JsonMapper.fromJsonString(rst.getMessage(), List.class));
            return new JsonResult(ResultCode.SUCCESS, rst);
        }
    }

    /**
     * 立即运行
     */
    @Override
    public JsonResult taskRunNowByTaskId(String taskId) throws Exception {
        boolean flag = false;
        //根据任务id，查询DMP资源导航树和DW系统配置信息
        Map<String, Object> map = offlineSynchDao.selectNavigationTreeByTaskId(taskId);
        if (map.size() > 0 && map != null) {
            flag = this.publish(map);
            if (flag) {
                return new JsonResult(ResultCode.SUCCESS, flag);
            } else {
                return new JsonResult(ResultCode.SYS_ERROR, flag);
            }
        } else {
            return new JsonResult(ResultCode.SYS_ERROR, flag);
        }
    }

    /**
     * 发布流程
     */
    private boolean publish(Map<String, Object> map) {
        String taskId = map.get("taskId").toString(); //任务id
        String projectId = map.get("projectId").toString(); //项目id
        String treeName = map.get("treeName").toString(); //流程名称
        String azkabanExectorXmlExec = map.get("azkabanExectorXmlExec").toString(); //执行数据同步任务命令
        String azkabanLocalTaskFilePath = map.get("azkabanLocalTaskFilePath").toString(); //文件路径
        String azkabanMonitorUrl = map.get("azkabanMonitorUrl").toString();//AZKABAN WEB服务地址

        String azkabanJobCommand = "command=" + azkabanExectorXmlExec + " " + projectId + " ${azkaban.flow.flowid} ${azkaban.job.id} ${azkaban.flow.execid} " + treeName;

        /**
         * 当前任务生成文件存放根路径
         */
        String localTaskPath = azkabanLocalTaskFilePath + "/" + projectId + "/local/" + taskId;
        File localTaskFile = new File(localTaskPath);
        if (!localTaskFile.exists()) {
            localTaskFile.mkdirs();
        }

        String localTaskSourcePath = localTaskPath + "/source/";
        File localTaskSourceFile = new File(localTaskSourcePath);
        if (!localTaskSourceFile.exists()) {
            localTaskSourceFile.mkdirs();
        }

        String localTaskExecArgsPath = localTaskPath + "/execArgs/";
        File localTaskExecArgsFile = new File(localTaskExecArgsPath);
        if (!localTaskExecArgsFile.exists()) {
            localTaskExecArgsFile.mkdirs();
        }

        String localTaskZipPath = localTaskPath + "/target/";
        File localTaskZipFile = new File(localTaskZipPath);
        if (!localTaskZipFile.exists()) {
            localTaskZipFile.mkdirs();
        }

        //先删除上次创建的文件 ---每次都会新建
        FileUtils.deleteDirectory(localTaskPath);


        List<String> contents = new ArrayList<>();

        // 子流程类型
        contents.add("type=command");
        contents.add(azkabanJobCommand);

        // 生成job文件
        String jobFileAbsolutePath = localTaskSourcePath + treeName + ".job";
        FileUtils.write(jobFileAbsolutePath, contents);

        String localZipTargetFileName = treeName + ".zip";
        ZipUtils.zip(localTaskSourcePath, localTaskZipPath, localZipTargetFileName);

        //上传到azkaban todo
        //上次zip包到azkaban
        String localTaskZipAbsolutePath = localTaskZipPath + "/" + localZipTargetFileName;
        AzkabanApiUtils2 azkabanApiUtils = new AzkabanApiUtils2(azkabanMonitorUrl);
        return azkabanApiUtils.loginCreateProjectuploadZipAndExecute("jz_localflow_" + projectId, "local_sync_project", localTaskZipAbsolutePath, treeName);
    }

    /**
     * 删除任务
     */
    @Override
    public JsonResult delTaskByTaskId(String taskId) throws Exception {
        //通过taskId，查询任务和资源是否存在
        Map map = dmpDevelopTaskDao.getDmpTaskAndTreeInfo(taskId);
        if (map.size() == 0 || map == null) {
            return new JsonResult(ResultCode.OPERATION_DATA_NO_EXIST);
        }
        if (StringUtils.isEmpty(map.get("treeId").toString())) {
            return new JsonResult(ResultCode.OPERATION_DATA_NO_EXIST);
        }
        dmpDevelopTaskDao.deleteTaskByTaskId(taskId);
        dmpDevelopTaskDao.deleteNavigationTreeByTreeId(map.get("treeId").toString());
        return new JsonResult(ResultCode.SUCCESS);
    }

    /**
     * 任务状态查看，异步调用azkaban服务
     */
    @Override
    public JsonResult<List<CheckTaskStatusPageDto>> queryCheckTaskStatusListPage(CheckTaskStatusPageReq checkTaskStatusPageReq) throws Exception {
        List<FlowExecution> list = new ArrayList<>();

        Long projectId = Long.valueOf(checkTaskStatusPageReq.getProjectId()); //项目id
        String treeName = checkTaskStatusPageReq.getTreeName(); //任务树名称
        DmpProjectSystemInfo publishToProjectSystemInfo = dmpProjectDao.queryProjectSystemInfo(projectId);

        //调用azkaban服务
        String azkabanApiUrl = publishToProjectSystemInfo.getAzkabanMonitorUrl();
        AzkabanApiUtils2 azkabanApiUtils = new AzkabanApiUtils2(azkabanApiUrl);
        list = azkabanApiUtils.getSyncingFlowExecution(projectId, treeName, checkTaskStatusPageReq.getPageNum(), checkTaskStatusPageReq.getPageSize());

        SimpleDateFormat dtf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        List<CheckTaskStatusPageDto> returnList = new ArrayList<>();
        if (list.size() > 0 && list != null) {
            for (FlowExecution str : list) {
                CheckTaskStatusPageDto dto = new CheckTaskStatusPageDto();
                long endTime = str.getEndTime();
                long startTime = str.getStartTime();
                dto.setCostTime(endTime - startTime); //耗时
                dto.setExecId(str.getExecId()); //任务执行id
                dto.setStatus(str.getStatus());
                dto.setSubmitUser(str.getSubmitUser());
                dto.setEndTime(dtf.format(new Date(endTime))); //结束时间
                dto.setStartTime(dtf.format(new Date(startTime))); //开始时间
                returnList.add(dto);
            }
        }
        return new JsonResult(ResultCode.SUCCESS, returnList);
    }

    @Override
    public PageInfoResponse<CheckJyRlueStatusDto> selectCheckJyStatusInfo(CheckJyRlueStatusReq checkJyRlueStatusReq) throws Exception {
        PageInfoResponse<CheckJyRlueStatusDto> pageInfoResponse = new PageInfoResponse<>();

        PageHelper.startPage(checkJyRlueStatusReq.getPageNum(), checkJyRlueStatusReq.getPageSize());
        List<CheckJyRlueStatusDto> list = offlineSynchDao.selectCheckJyStatusInfo(checkJyRlueStatusReq.getExecutionId());
        PageInfo<CheckJyRlueStatusDto> pageInfo = new PageInfo<>(list);

        pageInfoResponse.setCode(ResultCode.SUCCESS);
        pageInfoResponse.setMessage("查询成功");
        pageInfoResponse.setData(pageInfo);
        return pageInfoResponse;
    }

    @Override
    public JsonResult querySoureAndTargetColumnsByParams(SoureAndTargetColumnsReq soureAndTargetColumnsReq) throws Exception {
        //通过源数据库id ，查询数据源配置
        DmpAgentDatasourceInfo dsInfo = offlineSynchDao.querySourceDbInfoBySourceId(soureAndTargetColumnsReq.getSourceDbId());

        //通过目标数据库id ，查询数据配置
        DmpAgentDatasourceInfo targetInfo = offlineSynchDao.querySourceDbInfoBySourceId(soureAndTargetColumnsReq.getTargetDbId());

        //dsInfo.setHdfsUserName(body.getAccessId()); // 专为HDFS CLIENT提供
        dsInfo.setFileType(soureAndTargetColumnsReq.getFileType());
        dsInfo.setDelimiter(soureAndTargetColumnsReq.getCsvDelimiter());
        dsInfo.setIsHaveHeader(soureAndTargetColumnsReq.getCsvIsHaveHeader());

        //解码源数据库密码
        if (StringUtils.isNotBlank(dsInfo.getPassword())) {
            dsInfo.setPassword(new BaseService().decode(dsInfo.getPassword(), publicKey));
        }

        //创建jdbc,获取源数据表字段
        DmpAgentResult rst = dmpDsAgentServiceImp.getTableColumnList(dsInfo, soureAndTargetColumnsReq.getTargetTableName());
        if (!rst.getCode().val().equals("200")) {
            return new JsonResult(rst.getCode(), rst.getMessage());
        }
        List<Map<String, Object>> cols = (List<Map<String, Object>>) JsonMapper.fromJsonString(rst.getMessage(), List.class);
        if (cols == null || cols.size() == 0) {
            return new JsonResult(ResultCode.INTERNAL_SERVER_ERROR, "获取不到对应数据源的数据");
        }
        logger.info("######################################  数据源的表字段 success  ####################################");

        DmpTable table = new DmpTable(); //项目表信息
        table.setTableMode(dsInfo.getDatasourceTypeId()); //数据源类型ID
        table.setCnName(soureAndTargetColumnsReq.getTargetTableName()); //中文名
        table.setIsShow("0"); //是否隐藏
        table.setName(soureAndTargetColumnsReq.getTargetTableName());//表名
        table.setTableDesc("数据同步-源表"); //描述
        table.setDataStatus("1"); //数据表状态 默认1

        dmpTableDao.deleteTableColumnByName(table);
        dmpTableDao.deleteTableByName(table);
        dmpTableDao.insert(table);

        DmpTableFieldSchema schema = new DmpTableFieldSchema(); //项目表字段类型
        schema.setDsId(Integer.valueOf(dsInfo.getDatasourceTypeId()));
        List<DmpTableFieldSchema> sourceFieldSchema = dmpTableFieldSchemaDao.queryAll(schema); //源字段架构
        schema.setDsId(Integer.valueOf(dsInfo.getDatasourceTypeId()));
        List<DmpTableFieldSchema> targetFieldSchema = dmpTableFieldSchemaDao.queryAll(schema); //目标字段架构
        if (sourceFieldSchema != null && sourceFieldSchema.size() > 0) {
            Map<String, Integer> ftm = new HashMap<String, Integer>();
            for (DmpTableFieldSchema s : sourceFieldSchema) { //便利源字段类型
                ftm.put(s.getFieldType(), s.getId().intValue());
            }
            List<DmpTableColumn> columns = new ArrayList<DmpTableColumn>();
            for (int i = 0; i < cols.size(); i++) { //便利源字段
                Map<String, Object> map = cols.get(i);
                String name = (String) map.get("name");
                String type = (String) map.get("type");
                Integer size = (Integer) map.get("size");
                Integer scale = (Integer) map.get("scale");
                String comment = (String) map.get("comment");

                DmpTableColumn dtc = new DmpTableColumn();
                dtc.setField(name); //字段
                dtc.setType(ftm.get(type)); //类型
                if (size != null) dtc.setSize(size); //大小
                if (scale != null) dtc.setScale(scale); //经度
                dtc.setTableId(table.getId().intValue()); //tableID
                dtc.setComment(comment); //备注说明
                dtc.setDataStatus("1"); //数据表状态 默认1
                columns.add(dtc);
            }
            //向dmp_table_column批量新增
            dmpTableColumnDao.insertBatch(columns);
        }

        Map<String, Object> params = new HashMap<String, Object>();
        params.put("targetDsId", targetInfo.getDatasourceTypeId()); //数据源类型ID
        params.put("sourceDsId", dsInfo.getDatasourceTypeId());
        params.put("tableName", soureAndTargetColumnsReq.getTargetTableName());
        List<Map<String, Object>> fieldsMappingList = dmpTableFieldSchemaDao.getTableFieldsMapping(params);
        List<Map<String, Object>> sourceFieldColumns = new ArrayList<Map<String, Object>>();
        List<Map<String, Object>> targetFieldColumns = new ArrayList<Map<String, Object>>();
        for (Map<String, Object> m : fieldsMappingList) {
            String field = (String) m.get("field");
            String sourceType = (String) m.get("sourceType");
            String targetType = "STRING";
            if (m.get("targetType") != null && StringUtils.isNotBlank((String) m.get("targetType"))) {
                targetType = (String) m.get("targetType");
            }
            Integer size = (Integer) m.get("size");
            Integer scale = (Integer) m.get("scale");

            Map<String, Object> sourceFieldColumn = new HashMap<String, Object>();//源表字段
            sourceFieldColumn.put("name", field); //字段
            sourceFieldColumn.put("type", sourceType); //类型
            sourceFieldColumn.put("size", size); //大小
            sourceFieldColumn.put("scale", scale); //经度

            Map<String, Object> targetFieldColumn = new HashMap<String, Object>();//目标表字段
            targetFieldColumn.put("name", field);
            targetFieldColumn.put("type", targetType);
            targetFieldColumn.put("size", size);
            targetFieldColumn.put("scale", scale);

            sourceFieldColumns.add(sourceFieldColumn);
            targetFieldColumns.add(targetFieldColumn);
        }
        Map<String, Object> bak = new HashMap<String, Object>();
        //bak.put("sourceFieldSchema", sourceFieldSchema);
        bak.put("sourceFieldColumns", sourceFieldColumns);
        //bak.put("targetFieldSchema", targetFieldSchema);
        bak.put("targetFieldColumns", targetFieldColumns);

        dmpTableDao.deleteById(table.getId());
        dmpTableColumnDao.deleteByTableId(table.getId());
        return new JsonResult(ResultCode.SUCCESS, bak);
    }

    @Override
    public PageInfoResponse<DvRuleTDto> queryJyRuleListPage(BasePageBean basePageBean, HttpServletRequest httpRequest) throws Exception {
        PageInfoResponse<DvRuleTDto> pageInfoResponse = new PageInfoResponse<>();

        PageHelper.startPage(basePageBean.getPageNum(), basePageBean.getPageSize());
        DvRuleT dvRuleT = new DvRuleT();
        List<DvRuleTDto> list = dvRuleTDao.queryJyRuleListPage(dvRuleT);
        if (list.size() > 0 && list != null) {
            for(DvRuleTDto dto : list){
               dto.setRuleContentJson(JSON.parseObject(dto.getRuleContent()));
            }
        }
        PageInfo<DvRuleTDto> pageInfo = new PageInfo<>(list);

        pageInfoResponse.setCode(ResultCode.SUCCESS);
        pageInfoResponse.setMessage("查询成功");
        pageInfoResponse.setData(pageInfo);
        return pageInfoResponse;
    }

    @Override
    public JsonResult addSyncTask(SyncDmpTaskAddReq syncDmpTaskAddReq) throws Exception {
        return null;
    }

}
